#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QDateTime>
#include <QMessageBox>
#include <QDebug>
#include <QScrollBar>
#include <QEvent>

#ifdef PTU5
#define SERIAL_PORT "ttymxc2"
#else
#define SERIAL_PORT "ttyUSB0"
#endif

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
    , m_serial(new QSerialPort(this))
    , m_width(70) {

    this->setStatusBar(new QStatusBar(this));
    QFont f;
    f.setStyleHint(QFont::Monospace);
    f.setWeight(QFont::Bold);
    f.setFamily("Misc Fixed");
    f.setPixelSize(12);
    this->statusBar()->setFont(f);

    ui->setupUi(this);

    openSerialPort();

    //ui->updateProgress->setRange(0, 100);
    //ui->updateProgress->reset();

#if 0
    QStringList lst;
    QString start = QDateTime::currentDateTime().toString(Qt::ISODate);
    lst << QString("Start: ") + start.leftJustified(m_width-10);
    lst << QString("").leftJustified(m_width-3, '=');
    lst << QString("Update tool version: %1 - %2 %3").arg(APP_VERSION).arg(APP_BUILD_DATE).arg(APP_BUILD_TIME).leftJustified(m_width-3);
    lst << QString("Machine number     : %1 ").arg(m_worker->machineNr()).leftJustified(m_width-3);
    lst << QString("Customer number    : %1 ").arg(m_worker->customerNr()).leftJustified(m_width-3);
    lst << QString("Zone number        : %1 (%2)").arg(m_worker->zoneNr()).arg(Utils::zoneName(m_worker->zoneNr())).leftJustified(m_width-3);
    lst << QString("APISM version      : %1").arg(m_worker->apismVersion()).leftJustified(m_width-3);
    lst << QString("").leftJustified(m_width-3, '=');

    ui->updateStatus->setText(lst.join('\n'));
    ui->updateStatus->setEnabled(true);
    // ui->updateStatus->installEventFilter(this);

    m_startTimer = new QTimer(this);
    connect(m_startTimer, SIGNAL(timeout()), m_worker, SLOT(start()));
    m_startTimer->setSingleShot(true);
    m_startTimer->start(1000);

    m_exitTimer = new QTimer(this);
    connect(m_exitTimer, SIGNAL(timeout()), ui->exit, SLOT(click()));
    m_exitTimer->setSingleShot(true);
    m_exitTimer->start(1800 * 1000);

    connect(ui->exit, SIGNAL(clicked()),this,SLOT(onQuit()));
    connect(m_worker, SIGNAL(disableExit()),this,SLOT(onDisableExit()));
    connect(m_worker, SIGNAL(enableExit()),this,SLOT(onEnableExit()));
    connect(m_worker, SIGNAL(stopStartTimer()),this,SLOT(onStopStartTimer()));
    connect(m_worker, SIGNAL(restartExitTimer()),this,SLOT(onRestartExitTimer()));
    connect(m_worker, SIGNAL(appendText(QString,QString)),this,SLOT(onAppendText(QString,QString)));
    connect(m_worker, SIGNAL(showErrorMessage(QString,QString)),this, SLOT(onShowErrorMessage(QString,QString)));
    connect(m_worker, SIGNAL(showStatusMessage(QString,QString)),this, SLOT(onShowStatusMessage(QString,QString)));
    connect(m_worker, SIGNAL(showErrorMessage(QStringList)),this, SLOT(onShowErrorMessage(QStringList)));
    connect(m_worker, SIGNAL(showStatusMessage(QString,QString)),this, SLOT(onShowStatusMessage(QString,QString)));
    connect(m_worker, SIGNAL(replaceLast(QString,QString)),this,SLOT(onReplaceLast(QString,QString)));
    connect(m_worker, SIGNAL(replaceLast(QStringList,QString)),this, SLOT(onReplaceLast(QStringList,QString)));
#endif
}

MainWindow::~MainWindow() {
    //delete m_startTimer;
    //delete m_exitTimer;
    closeSerialPort();
    delete ui;
}

bool MainWindow::openSerialPort() {
   // const SettingsDialog::Settings p = m_settings->settings();
   m_serial->setPortName(SERIAL_PORT);
   m_serial->setBaudRate(QSerialPort::Baud115200);
   m_serial->setDataBits(QSerialPort::DataBits::Data8);
   m_serial->setParity(QSerialPort::Parity::NoParity);
   m_serial->setStopBits(QSerialPort::StopBits::OneStop);
   m_serial->setFlowControl(QSerialPort::FlowControl::NoFlowControl);
   if (m_serial->open(QIODevice::ReadWrite)) {
       //m_console->setEnabled(true);
       //m_console->setLocalEchoEnabled(p.localEchoEnabled);
       //m_ui->actionConnect->setEnabled(false);
       //m_ui->actionDisconnect->setEnabled(true);
       //m_ui->actionConfigure->setEnabled(false);
       //showStatusMessage(tr("Connected to %1 : %2, %3, %4, %5, %6")
       //                  .arg(p.name, p.stringBaudRate, p.stringDataBits,
       //                       p.stringParity, p.stringStopBits, p.stringFlowControl));
       return true;
   } else {
       //QMessageBox::critical(this, tr("Error"), m_serial->errorString());

       //showStatusMessage(tr("Open error"));
   }
   return false;
}

bool MainWindow::closeSerialPort() {
    if (m_serial->isOpen()) {
        m_serial->close();
        return true;
    }
    return false;
    //m_console->setEnabled(false);
    //m_ui->actionConnect->setEnabled(true);
    //m_ui->actionDisconnect->setEnabled(false);
    //m_ui->actionConfigure->setEnabled(true);
    //showStatusMessage(tr("Disconnected"));
}

#if 0
void MainWindow::customEvent(QEvent *event) {
    if (event->type() == ProgressEvent::type()) {
        ProgressEvent *pevent = (ProgressEvent *)event;
        int const progress = pevent->progressPercent();
        QObject const *sender = pevent->sender();
        if (sender == this) {
            switch(progress) {
            case 0: {
                ui->updateProgress->reset();
            } break;
            case START_PROGRESS_LOOP: {
                m_progressRunning = true;
                ui->updateProgress->reset();
                // m_progressValue = 10;
                QApplication::postEvent(this, new ProgressEvent(this, 1));
            } break;
            case STOP_PROGRESS_LOOP: {
                m_progressRunning = false;
                // m_progressValue -= 10;
                // m_worker->setProgress(m_progressValue/10);
            } break;
            default: {
                if (m_progressRunning) {
                    // m_progressValue = progress;
                    ui->updateProgress->setValue(progress);
                    // ueberpruefen: hauptfenster schickt sich selber ein event
                    // QApplication::postEvent(this, new ProgressEvent(this, progress));
                    // QThread::msleep(500);
                }}
            }
        } else
        if (sender == m_worker) {
            switch(progress) {
            case 0: {
                ui->updateProgress->reset();
            } break;
            case START_PROGRESS_LOOP: {
                QApplication::postEvent(this, new ProgressEvent(this, START_PROGRESS_LOOP));
            } break;
            case STOP_PROGRESS_LOOP: {
                QApplication::postEvent(this, new ProgressEvent(this, STOP_PROGRESS_LOOP));
            } break;
            default:{
                ui->updateProgress->setValue(progress);
            }}
        } else {
            qCritical() << "!!! UNKNOWN SENDER !!!";
        }
    }

    QThread::yieldCurrentThread();
}

void MainWindow::onStopStartTimer() {
    m_startTimer->stop();
}

void MainWindow::onDisableExit() {
   ui->exit->setEnabled(false);
}

void MainWindow::onEnableExit() {
   ui->exit->setEnabled(true);
}

void MainWindow::onRestartExitTimer() {
    m_exitTimer->stop();
    m_exitTimer->start(5 * 1000);

    scrollDownTextEdit();
    ui->updateStatus->setEnabled(false);
}

void MainWindow::onQuit() {
    m_exitTimer->stop();
    int errorCode = 0;

    qCritical()
        << QString("ON QUIT: CURRENT STEP %1")
            .arg(m_worker->getSmap()[m_worker->currentStep()]);

    // TODO: replace SEND_LAST_VERSION with UPDATE_SUCCEEDED
    if (m_worker->currentStep() != Worker::UPDATE_STEP::SEND_LAST_VERSION) {
        errorCode = -1;
    }
    qCritical() << QString("ON QUIT: EXIT CODE %1").arg(errorCode);
    qApp->exit(errorCode);
}

void MainWindow::scrollDownTextEdit() {
    // Utils::printInfoMsg(QString("SCROLL-DOWN-TEXT_EDIT CALLED AT ")
    //    + QDateTime::currentDateTime().toString(Qt::ISODateWithMs));

    ui->updateStatus->setEnabled(true);

    QTextCursor tmpCursor = ui->updateStatus->textCursor();
    tmpCursor.movePosition(QTextCursor::End);
    ui->updateStatus->setTextCursor(tmpCursor);
    ui->updateStatus->ensureCursorVisible();
}

void MainWindow::onAppendText(QString text, QString suffix) {
    // Utils::printInfoMsg(QString("ON APPEND CALLED AT ")
    //    + QDateTime::currentDateTime().toString(Qt::ISODateWithMs));

    QString editText = ui->updateStatus->toPlainText();
    scrollDownTextEdit();

    if (!suffix.isNull() && suffix.size() > 0) {
        //qInfo() << "TEXT" << text << "SUFFIX" << suffix;
        if (suffix == Worker::UPDATE_STEP_SUCCESS || suffix == Worker::UPDATE_STEP_FAIL) {
            ui->updateStatus->insertPlainText(QString("\n").leftJustified(m_width-3, '=') + " ");
            // editText += QString("\n").leftJustified(m_width-3, '=');
            // editText += " ";
        }
        QString const &add = (QString("\n") + text).leftJustified(m_width - (2 + suffix.size())) + suffix;
        ui->updateStatus->insertPlainText(add);
        // editText += add;
    } else {
        QString const &add = text.leftJustified(m_width-9);
        ui->updateStatus->insertPlainText(add);
        //editText += add;
    }

    // debug
    // QString editText = ui->updateStatus->toPlainText();
    // Utils::printLineEditInfo(editText.split('\n', QString::SplitBehavior::SkipEmptyParts));
    // ui->updateStatus->setText(editText.trimmed());

    scrollDownTextEdit();
}

void MainWindow::onReplaceLast(QStringList newTextLines, QString suffix) {
    // Utils::printInfoMsg(QString("ON REPLACE LAST (LIST) CALLED AT ")
    //    + QDateTime::currentDateTime().toString(Qt::ISODateWithMs));
    int const s = newTextLines.size();
    if (s > 0) {
        QString editText = ui->updateStatus->toPlainText();
        QStringList lines = editText.split('\n', QString::SplitBehavior::SkipEmptyParts);
        QString newText;
        if (lines.size() >= s) {
            for (int i = 0; i < s; ++i) {
                lines.removeLast();
            }
            if (lines.size() > 0) {
                newText = lines.join('\n');
                newText += '\n';
            }
            QStringList newLines;
            for (int i = 0; i < s; ++i) {
                if (i == 0 && !suffix.isNull() && suffix.size() > 0 && suffix != "\n") {
                    newLines += Utils::rstrip(newTextLines.at(i).leftJustified(m_width-10) + suffix);
                } else {
                    newLines += Utils::rstrip(newTextLines.at(i).leftJustified(m_width-10));
                }
            }
            lines += newLines;
            newText += newLines.join(' ');
        }

        ui->updateStatus->setText(newText);
        Utils::printLineEditInfo(lines);
        scrollDownTextEdit();
    }
}

void MainWindow::onReplaceLast(QString text, QString suffix) {
    // Utils::printInfoMsg(QString("ON REPLACE LAST (TEXT) CALLED AT ")
    //    + QDateTime::currentDateTime().toString(Qt::ISODateWithMs));
    QString editText = ui->updateStatus->toPlainText();
    QStringList lines = editText.split('\n', QString::SplitBehavior::SkipEmptyParts);
    if (lines.size() > 0) {
        // removing the last line is really meant for refreshing the last line
        // with a string very similar than the original one, typically only
        // followed by a suffix.
        if (lines.last().contains(text)) {
            lines.removeLast();
        }
        if (!suffix.isNull() && suffix.size() > 0 && suffix != "\n") {
            QString const add = text.leftJustified(m_width-10) + suffix;
            if (!add.isEmpty()) {
                lines += text.leftJustified(m_width-10) + suffix;
            }
        } else {
            QString const add = text.leftJustified(m_width-10);
            if (!add.isEmpty()) {
                lines += text.leftJustified(m_width-10);
            }
        }
    }

    Utils::printLineEditInfo(lines);
    ui->updateStatus->setText(lines.join('\n').trimmed());
    scrollDownTextEdit();
}

void MainWindow::onShowMessage(QString title, QString text) {
    this->statusBar()->clearMessage();
    this->statusBar()->showMessage( // timeout: 10000
        QString(title + " " + text).leftJustified(80, ' '), 10000);
}

void MainWindow::onShowErrorMessage(QString title, QString text) {
    onShowMessage(title, text);
}

void MainWindow::onShowStatusMessage(QString title, QString text) {
    onShowMessage(title, text);
}

void MainWindow::onShowErrorMessage(QStringList lst) {
    if (lst.size() >= 2) {
        onShowMessage(lst.at(0), lst.at(1));
    }
    if (lst.size() == 1) {
        onShowMessage(lst.at(0), "");
    }
}

void MainWindow::onShowStatusMessage(QStringList lst) {
    if (lst.size() >= 2) {
        onShowMessage(lst.at(0), lst.at(1));
    }
    if (lst.size() == 1) {
        onShowMessage(lst.at(0), "");
    }
}
#endif
